# -*- coding:utf-8 -*-

__version__ = '0.1.6'
__author__ = 'TabQ'


from gugu.base import Base

from gugu.billboard import BillBoard

from gugu.classify import Classify

from gugu.lowriskintarb import LowRiskIntArb

from gugu.macro import Macro

from gugu.marketdata import MarketData

from gugu.reference import Reference

from gugu.stockdata import StockData

from gugu.stockinfo import StockInfo

from gugu.utility import Utility

